@@ -4,9 +4,9 @@
  *	This module implements general commands of a table widget,
  *	based on the major/minor command structure.
  *
- * Copyright (c) 1998-1999 Jeffrey Hobbs
+ * Copyright (c) 1998-2002 Jeffrey Hobbs
  *
- * See the file "license.terms" for information on usage and redistribution
+ * See the file "license.txt" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
  */
@@ -106,7 +106,8 @@ Table_ActivateCmd(ClientData clientData, register Tcl_Interp *interp,
 		/* no error checking because GetIndex did it for us */
 		p++;
 		x = strtol(p, &p, 0) - x - tablePtr->activeX;
-		y = strtol(++p, &p, 0) - y - tablePtr->activeY;
+		p++;
+		y = strtol(p, &p, 0) - y - tablePtr->activeY;
 
 		textLayout = Tk_ComputeTextLayout(tagPtr->tkfont,
 					tablePtr->activeBuf, -1,
@@ -115,14 +116,14 @@ Table_ActivateCmd(ClientData clientData, register Tcl_Interp *interp,
 
 		tablePtr->icursor = Tk_PointToChar(textLayout, x, y);
 		Tk_FreeTextLayout(textLayout);
-		TableConfigCursor(tablePtr);
+		TableRefresh(tablePtr, row, col, CELL|INV_FORCE);
 	    }
 	}
 	tablePtr->flags |= HAS_ACTIVE;
     }
     return result;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -228,7 +229,7 @@ Table_AdjustCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -304,8 +305,8 @@ Table_BboxCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
-static char *bdCmdNames[] = {
+
+static CONST char *bdCmdNames[] = {
     "mark", "dragto", (char *)NULL
 };
 enum bdCmd {
@@ -435,10 +436,9 @@ Table_BorderCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
-
+
 /* clear subcommands */
-static char *clearNames[] = {
+static CONST char *clearNames[] = {
     "all", "cache", "sizes", "tags", (char *)NULL
 };
 enum clearCommand {
@@ -507,7 +507,7 @@ Table_ClearCmd(ClientData clientData, register Tcl_Interp *interp,
 	}
 
 	if (cmdIndex == CLEAR_CACHE || cmdIndex == CLEAR_ALL) {
-	    Tcl_DeleteHashTable(tablePtr->cache);
+	    Table_ClearHashTable(tablePtr->cache);
 	    Tcl_InitHashTable(tablePtr->cache, TCL_STRING_KEYS);
 	    /* If we were caching and we have no other data source,
 	     * invalidate all the cells */
@@ -519,7 +519,7 @@ Table_ClearCmd(ClientData clientData, register Tcl_Interp *interp,
     } else {
 	int row, col, r1, r2, c1, c2;
 	Tcl_HashEntry *entryPtr;
-	char buf[INDEX_BUFSIZE];
+	char buf[INDEX_BUFSIZE], *value;
 
 	if (TableGetIndexObj(tablePtr, objv[3], &row, &col) != TCL_OK ||
 	    ((objc == 5) &&
@@ -587,6 +587,8 @@ Table_ClearCmd(ClientData clientData, register Tcl_Interp *interp,
 
 		if ((cmdIndex == CLEAR_CACHE || cmdIndex == CLEAR_ALL) &&
 		    (entryPtr = Tcl_FindHashEntry(tablePtr->cache, buf))) {
+		    value = (char *) Tcl_GetHashValue(entryPtr);
+		    if (value) { ckfree(value); }
 		    Tcl_DeleteHashEntry(entryPtr);
 		    /* if the cache is our data source,
 		     * we need to invalidate the cells changed */
@@ -611,7 +613,7 @@ Table_ClearCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -662,17 +664,19 @@ Table_CurselectionCmd(ClientData clientData, register Tcl_Interp *interp,
 	    TableRefresh(tablePtr, row, col, CELL);
 	}
     } else {
+	Tcl_Obj *objPtr = Tcl_NewObj();
+
 	for (entryPtr = Tcl_FirstHashEntry(tablePtr->selCells, &search);
 	     entryPtr != NULL; entryPtr = Tcl_NextHashEntry(&search)) {
-	    Tcl_AppendElement(interp,
-			      Tcl_GetHashKey(tablePtr->selCells, entryPtr));
-
+	    value = Tcl_GetHashKey(tablePtr->selCells, entryPtr);
+	    Tcl_ListObjAppendElement(NULL, objPtr,
+				     Tcl_NewStringObj(value, -1));
 	}
-	Tcl_ArgResult(interp,TableCellSort(tablePtr, Tcl_GetResult(interp)));
+	Tcl_SetObjResult(interp, TableCellSortObj(interp, objPtr));
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -696,7 +700,7 @@ Table_CurvalueCmd(ClientData clientData, register Tcl_Interp *interp,
     register Table *tablePtr = (Table *) clientData;
 
     if (objc > 3) {
-	Tcl_WrongNumArgs(interp, 2, objv, "curvalue ?<value>?");
+	Tcl_WrongNumArgs(interp, 2, objv, "?<value>?");
 	return TCL_ERROR;
     } else if (!(tablePtr->flags & HAS_ACTIVE)) {
 	return TCL_OK;
@@ -707,7 +711,7 @@ Table_CurvalueCmd(ClientData clientData, register Tcl_Interp *interp,
 	int len;
 
 	value = Tcl_GetStringFromObj(objv[2], &len);
-	if (strcmp(value, tablePtr->activeBuf) == 0) {
+	if (STREQ(value, tablePtr->activeBuf)) {
 	    Tcl_SetObjResult(interp, objv[2]);
 	    return TCL_OK;
 	}
@@ -734,7 +738,7 @@ Table_CurvalueCmd(ClientData clientData, register Tcl_Interp *interp,
     Tcl_SetStringObj(Tcl_GetObjResult(interp), tablePtr->activeBuf, -1);
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -756,7 +760,6 @@ Table_GetCmd(ClientData clientData, register Tcl_Interp *interp,
 	     int objc, Tcl_Obj *CONST objv[])
 {
     register Table *tablePtr = (Table *) clientData;
-    Tcl_Obj *resultPtr = Tcl_GetObjResult(interp);
     int result = TCL_OK;
     int r1, c1, r2, c2, row, col;
     if (objc < 3 || objc > 4) {
@@ -765,33 +768,27 @@ Table_GetCmd(ClientData clientData, register Tcl_Interp *interp,
     } else if (TableGetIndexObj(tablePtr, objv[2], &row, &col) == TCL_ERROR) {
 	result = TCL_ERROR;
     } else if (objc == 3) {
-        /* Copy the cell contents to another string, 
-	     otherwise the perltk Tcl_SetStringObj appears to
-	     clobber the string contents */
-    	char *contents;
-    	char *val;
-	contents = TableGetCellValue(tablePtr, row, col);
-	val = (char *)ckalloc(strlen(contents)+1);
-	strcpy(val, contents);
-	Tcl_SetStringObj(resultPtr, val, -1);
+	Tcl_SetObjResult(interp,
+		Tcl_NewStringObj(TableGetCellValue(tablePtr, row, col), -1));
     } else if (TableGetIndexObj(tablePtr, objv[3], &r2, &c2) == TCL_ERROR) {
 	result = TCL_ERROR;
     } else {
-	Tcl_Obj *objPtr;
+	Tcl_Obj *objPtr = Tcl_NewObj();
 
 	r1 = MIN(row,r2); r2 = MAX(row,r2);
 	c1 = MIN(col,c2); c2 = MAX(col,c2);
 	for ( row = r1; row <= r2; row++ ) {
 	    for ( col = c1; col <= c2; col++ ) {
-		objPtr = Tcl_NewStringObj(TableGetCellValue(tablePtr,
-			row, col), -1);
-		Tcl_ListObjAppendElement(NULL, resultPtr, objPtr);
+		Tcl_ListObjAppendElement(NULL, objPtr,
+			Tcl_NewStringObj(TableGetCellValue(tablePtr,
+				row, col), -1));
 	    }
 	}
+	Tcl_SetObjResult(interp, objPtr);
     }
     return result;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -855,7 +852,7 @@ Table_ScanCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -900,7 +897,7 @@ Table_SelAnchorCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -931,7 +928,7 @@ Table_SelClearCmd(ClientData clientData, register Tcl_Interp *interp,
 	Tcl_WrongNumArgs(interp, 3, objv, "all|<first> ?<last>?");
 	return TCL_ERROR;
     }
-    if (strcmp(Tcl_GetString(objv[3]), "all") == 0) {
+    if (STREQ(Tcl_GetString(objv[3]), "all")) {
 	Tcl_HashSearch search;
 	for(entryPtr = Tcl_FirstHashEntry(tablePtr->selCells, &search);
 	    entryPtr != NULL; entryPtr = Tcl_NextHashEntry(&search)) {
@@ -991,7 +988,7 @@ CLEAR_CELLS:
     if (key) goto CLEAR_BOTH;
     return result;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1028,8 +1025,7 @@ Table_SelIncludesCmd(ClientData clientData, register Tcl_Interp *interp,
     }
     return TCL_OK;
 }
-
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1136,7 +1132,7 @@ SET_CELLS:
     }
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1268,8 +1264,8 @@ Table_ViewCmd(ClientData clientData, register Tcl_Interp *interp,
 
     return TCL_OK;
 }
-
-
+
+#if 0
 /*
  *--------------------------------------------------------------
  *
@@ -1295,4 +1291,4 @@ Table_Cmd(ClientData clientData, register Tcl_Interp *interp,
 
     return result;
 }
-
+#endif
